'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _prettyjson = require('prettyjson');

var _prettyjson2 = _interopRequireDefault(_prettyjson);

var _friendlyErrorsWebpackPlugin = require('friendly-errors-webpack-plugin');

var _friendlyErrorsWebpackPlugin2 = _interopRequireDefault(_friendlyErrorsWebpackPlugin);

var _progressBarWebpackPlugin = require('progress-bar-webpack-plugin');

var _progressBarWebpackPlugin2 = _interopRequireDefault(_progressBarWebpackPlugin);

var _npmAutoInstallWebpackPlugin = require('npm-auto-install-webpack-plugin');

var _npmAutoInstallWebpackPlugin2 = _interopRequireDefault(_npmAutoInstallWebpackPlugin);

var _Loader = require('./Loader');

var _Loader2 = _interopRequireDefault(_Loader);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Builder
 *
 * Utility to ease the construction of webpack configurations
 */
var Builder = function () {

	/**
  * Constructor
  * 
  * Setup our expected properties and empty
  * webpack configuration
  * 
  * @constructor
  */
	function Builder() {
		var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
		var pkg = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

		_classCallCheck(this, Builder);

		this.pkg = pkg;

		if (!this.pkg) {
			this.pkg = require(_path2.default.join(process.cwd(), 'package.json'));
		}

		this._loaders = [];

		this.config = _lodash2.default.merge({
			debug: false,
			autoInstall: false,
			friendlyErrors: true,
			progressBar: true,
			env: null
		}, config);

		var OUTPUT_PATH = config.outputFolder || _path2.default.join(process.cwd(), 'public');

		this.webpackConfig = {
			target: 'web',
			entry: {},
			output: {
				path: OUTPUT_PATH,
				filename: '[name]'
			},
			resolve: {
				extensions: ['', '.js']
			},
			module: {
				loaders: []
			},
			plugins: []
		};

		if (config.publicPath) {
			this.webpackConfig.output.publicPath = config.publicPath;
		}

		this._requireLoaderDir(_path2.default.join(__dirname, 'loaders'));
	}

	/**
  * Convert to Webpack config object
  *
  * This function will first ensure all dependencies
  * are installed to complete a successful build.
  *
  * Then it will apply all of our loaders that were
  * defined through the Builder API.
  *
  * Then it will return a generated webpack.config
  *
  * this function is intended to be exported
  * 		module.exports = builder.toWebpack()
  * 
  * @return {Object} webpack.config
  */


	_createClass(Builder, [{
		key: 'toWebpack',
		value: function toWebpack() {
			// if the user has not provided us with an environment
			// we are going to assume that it is development
			if (!this.config.env) {
				this.development();
			}

			if (!this._verifyDependencies()) {
				return null;
			}

			for (var i = 0; i < this._loaders.length; i++) {
				this._loaders[i].applyLoader();
			}

			this._addSystemConfig();

			if (this.config.debug) {
				console.log(_prettyjson2.default.render(this.webpackConfig));
			}

			return this.webpackConfig;
		}
	}, {
		key: 'debug',
		value: function debug() {
			var bool = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

			this.config.debug = !!bool;
			return this;
		}

		/**
   * Add development-specific settings
   */

	}, {
		key: 'development',
		value: function development() {
			this.config.env = 'development';
			this.webpackConfig.devtool = this.webpackConfig.devtool || 'eval';
			return this;
		}

		/**
   * Add production-specific settings
   */

	}, {
		key: 'production',
		value: function production() {
			this.config.env = 'production';
			this.webpackConfig.devtool = 'cheap-module-source-map';
			this.webpackConfig.plugins.push(new _webpack2.default.DefinePlugin({
				'process.env': {
					'NODE_ENV': JSON.stringify('production')
				}
			}));
			return this;
		}

		/**
   * Set the build devtool
   * 
   * @param  {string} devtool new build devtool
   * @return {Builder}        chainable interface
   */

	}, {
		key: 'devtool',
		value: function devtool(_devtool) {
			this.webpackConfig.devtool = _devtool;
			return this;
		}

		/**
   * Set the build target
   * 
   * @param  {string} target new build target
   * @return {Builder}        chainable interface
   */

	}, {
		key: 'target',
		value: function target(_target) {
			this.webpackConfig.target = _target;
			return this;
		}

		/**
   * Add an entry param
   * 
   * @param  {string} src  path to output file
   * @param  {string} dist path to source file
   * @return {Builder}      chainable interface
   */

	}, {
		key: 'entry',
		value: function entry(src, dist) {
			this.webpackConfig.entry[src] = dist;
			return this;
		}

		/**
   * Add a plugin to the webpack configuration
   * 
   * @param  {mixed} plugin Plugin to add
   * @return {Builder}        chainable interface
   */

	}, {
		key: 'plugin',
		value: function plugin(_plugin) {
			this.webpackConfig.plugins.push(_plugin);
			return this;
		}

		/**
   * Returns the name of the current script that is being run via "npm run ..."
   * "npm run build" would return "build"
   *
   * useful for determining when to apply production|development specific settings
   */

	}, {
		key: 'npmScriptName',
		value: function npmScriptName() {
			return process.env.npm_lifecycle_event;
		}
	}, {
		key: 'middleware',
		value: function middleware() {}

		/**
   * Require all the loaders in a given directory
   *
   * This function will take a given path to a folder,
   * then loop over each file in that directory.
   * 
   * For each file that it finds, it will do 2 things:
   * - add the config-manipulator to the "Loader" prototype.
   * - add the accessor interface to the "Builder" instance
   *
   * The accessor handles dealing with starting new Loader
   * chains.  These chains are then processed and applied
   * to the webpack configuration after all loaders have
   * been defined.
   * 
   * @param  {string} dir folder location
   * @return {null}
   */

	}, {
		key: '_requireLoaderDir',
		value: function _requireLoaderDir(dir) {
			var _this = this;

			_fs2.default.readdirSync(dir).forEach(function (file) {
				var blacklist = ['.', '..', 'README.md', 'index.js'];

				if (blacklist.indexOf(file) < 0) {
					(function () {
						var name = file.replace('.js', '');
						var loader = require(_path2.default.join(dir, file));

						if (typeof loader === 'function') {
							_Loader2.default.prototype[name] = function () {
								for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
									args[_key] = arguments[_key];
								}

								this._handleArgs.apply(this, args);
								return loader.call.apply(loader, [this].concat(args));
							};

							_this[name] = function () {
								var _ref;

								var l = (_ref = new _Loader2.default(_this))[name].apply(_ref, arguments);
								_this._loaders.push(l);
								return l;
							};
						}
					})();
				}
			});
		}

		/**
   * Verify existence of dependencies
   *
   * This function will search through all of the dependencies needed
   * by the webpack configuration.  These dependencies are then checked
   * against the "package.json" file to see if they are installed.  If
   * they are missing, one of 2 things will happen.
   *
   * 1) If "autoInstall" is set to true, it will automatically add the
   * 		dependencies to the "package.json" file, which will then
   * 		trigger the NpmAutoInstallPlugin to install them before
   * 		the next webpack build.
   * 		
   * 2) If "autoInstall" is set to false.  The Builder will report back
   * 		to the user which dependencies are missing, and provide a
   * 		one-liner npm install to add the missing dependencies
   * 		
   * @return {boolean} whether or not all dependencies are installed
   */

	}, {
		key: '_verifyDependencies',
		value: function _verifyDependencies() {
			var missing = [];

			for (var i = 0; i < this._loaders.length; i++) {
				var pkgDeps = this.pkg.dependencies || {};
				var pkgDevDeps = this.pkg.devDependencies || {};
				var deps = this._loaders[i].definition.dependencies;

				for (var j = 0; j < deps.length; j++) {
					var dep = deps[j];
					if (!dep) {
						continue;
					}

					if (!pkgDeps[dep] && !pkgDevDeps[dep]) {
						missing.push(dep);
					}
				}
			}

			if (!missing.length) {
				return true;
			}

			// if(this.config.autoInstall) {
			// 	for(let i = 0; i < missing.length; i++) {
			// 		this.pkg.devDependencies[missing[i]] = "latest"
			// 	}
			// 	save package.json
			// 	return true
			// }

			console.log('You are missing ' + missing.length + ' dependencies, to install run:');
			console.log('');
			console.log('  npm install --save-dev ' + missing.join(' '));
			console.log('');
			return false;
		}

		/**
   * Add our system configuration
   *
   * this is just our plugins that we use to provide the
   * end-user with a better experience
   */

	}, {
		key: '_addSystemConfig',
		value: function _addSystemConfig() {
			this.config.progressBar && this._addProgressBar();
			this.config.friendlyErrors && this._addFriendlyErrors();
			this.config.autoInstall && this._addAutoInstall();
		}

		/**
   * Add the FriendlyErrorsWebpackPlugin to clean up the reporting
   * provided by Webpack
   */

	}, {
		key: '_addFriendlyErrors',
		value: function _addFriendlyErrors() {
			this.webpackConfig.devServer = this.webpackConfig.devServer || {};
			this.webpackConfig.devServer.quiet = true;
			this.webpackConfig.devMiddleware = this.webpackConfig.devMiddleware || {};
			this.webpackConfig.devMiddleware.logs = function () {};

			this.webpackConfig.quiet = true;
			this.webpackConfig.debug = false;
			this.webpackConfig.stats = {
				hash: false,
				version: false,
				timings: false,
				assets: false,
				chunks: false,
				modules: false,
				reasons: false,
				children: false,
				source: false,
				errors: false,
				errorDetails: false,
				warnings: false,
				publicPath: false
			};

			this.plugin(new _friendlyErrorsWebpackPlugin2.default());
		}

		/**
   * Add a progress bar to the build process for webpack
   */

	}, {
		key: '_addProgressBar',
		value: function _addProgressBar() {
			this.plugin(new _progressBarWebpackPlugin2.default({
				format: 'building:' + this.config.env + ' [' + _chalk2.default.green(':bar') + '] ' + _chalk2.default.green.bold(':percent') + ' [:elapsed]s',
				summary: false
			}));
		}

		/**
   * Add the auto-installer to watch "package.json" and install
   * missing dependencies automatically
   */

	}, {
		key: '_addAutoInstall',
		value: function _addAutoInstall() {
			this.plugin(new _npmAutoInstallWebpackPlugin2.default());
		}
	}]);

	return Builder;
}();

exports.default = Builder;