'use strict';

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _test = require('../../../src/test');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* global describe, beforeEach, it */

describe('(loader) - sass|scss', function () {
	var builder = void 0;

	beforeEach(function (done) {
		builder = new _test.Builder({}, {
			devDependencies: _test.FULL_PKG
		});

		done();
	});

	it('should set entry point', function (done) {
		builder.sass('foo.css', 'bar.scss');
		var conf = builder.toWebpack();
		_assert2.default.equal('bar.scss', conf.entry['foo.css']);

		done();
	});

	it('should add extensions', function (done) {
		builder.sass();
		var conf = builder.toWebpack();

		_assert2.default.ok(conf.resolve.extensions.indexOf('.sass') >= 0);
		_assert2.default.ok(conf.resolve.extensions.indexOf('.scss') >= 0);

		done();
	});

	it('should add all loaders in correct order', function (done) {
		builder.sass();
		var conf = builder.toWebpack();

		_assert2.default.equal('style-loader', conf.module.loaders[0].loaders[0]);
		_assert2.default.equal('css-loader', conf.module.loaders[0].loaders[1]);
		_assert2.default.equal('sass-loader', conf.module.loaders[0].loaders[2]);

		done();
	});

	it('if combined with postcss should still be in correct order', function (done) {
		builder.sass().postcss();
		var conf = builder.toWebpack();

		_assert2.default.equal('style-loader', conf.module.loaders[0].loaders[0]);
		_assert2.default.equal('css-loader', conf.module.loaders[0].loaders[1]);
		_assert2.default.equal('sass-loader', conf.module.loaders[0].loaders[2]);
		_assert2.default.equal('postcss-loader', conf.module.loaders[0].loaders[3]);

		done();
	});
});