'use strict';

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _ = require('.');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* global describe, beforeEach, it */

describe('Builder', function () {

	var builder = void 0;

	beforeEach(function (done) {
		builder = new _.Builder({}, {
			devDependencies: _.FULL_PKG
		});

		done();
	});

	it('should export an object', function (done) {
		builder.babel('foo.js', 'bar.js');
		_assert2.default.equal(true, (0, _lodash.isObject)(builder.toWebpack()));
		done();
	});

	it('should set output path', function (done) {
		builder = new _.Builder({
			outputFolder: 'test'
		});
		_assert2.default.equal('test', builder.webpackConfig.output.path);
		done();
	});

	it('should use cwd as default output path', function (done) {
		_assert2.default.equal(_path2.default.join(process.cwd(), 'public'), builder.webpackConfig.output.path);
		done();
	});

	it('should initiate loaders', function (done) {
		_assert2.default.equal(true, (0, _lodash.isFunction)(builder.babel));
		_assert2.default.equal(true, (0, _lodash.isFunction)(builder.coffee));
		_assert2.default.equal(true, (0, _lodash.isFunction)(builder.eslint));
		_assert2.default.equal(true, (0, _lodash.isFunction)(builder.jade));
		_assert2.default.equal(true, (0, _lodash.isFunction)(builder.json));
		_assert2.default.equal(true, (0, _lodash.isFunction)(builder.less));
		_assert2.default.equal(true, (0, _lodash.isFunction)(builder.postcss));
		_assert2.default.equal(true, (0, _lodash.isFunction)(builder.pug));
		_assert2.default.equal(true, (0, _lodash.isFunction)(builder.raw));
		_assert2.default.equal(true, (0, _lodash.isFunction)(builder.sass));
		_assert2.default.equal(true, (0, _lodash.isFunction)(builder.stylus));
		_assert2.default.equal(true, (0, _lodash.isFunction)(builder.swig));
		done();
	});

	// it('should fail to build without all deps', done => {
	// 	builder = new Builder({}, {
	// 		devDependencies: PARTIAL_PKG,
	// 	})
	// 	builder.stylus();
	// 	assert.ok(! builder.toWebpack());

	// 	done();
	// });

	it('should add system plugins', function (done) {
		var conf = builder.toWebpack();
		_assert2.default.equal(2, conf.plugins.length);

		done();
	});

	it('should set "devtool"', function (done) {
		builder.devtool('source-map');
		var conf = builder.toWebpack();
		_assert2.default.equal('source-map', conf.devtool);

		done();
	});

	it('should set "target"', function (done) {
		builder.target('node');
		var conf = builder.toWebpack();
		_assert2.default.equal('node', conf.target);

		done();
	});

	it('should add a plugin', function (done) {
		builder.plugin('test');
		var conf = builder.toWebpack();
		_assert2.default.equal(true, conf.plugins.indexOf('test') >= 0);
		done();
	});

	it('should add an entry point', function (done) {
		builder.entry('foo.custom', 'foo.source');
		var conf = builder.toWebpack();
		_assert2.default.equal('foo.source', conf.entry['foo.custom']);
		done();
	});
});